import os
import re

package_path = os.path.dirname(os.path.abspath(__file__))
ABBR_FILE = os.path.join(package_path, 'abbreviations_general.txt')
ABBR_NONENDING_FILE = os.path.join(package_path, 'abbreviations_nonending.txt')

ALPHABETIC = '[A-Za-záéíóúýðþæöÁÉÍÓÚÝÐÞÆÖ]+'
UPPER_CASE = '[A-ZÁÉÍÓÚÝÐÞÆÖ]'
# we can't use \\w because it only takes ascii chars into account
WORD_CHAR = '[A-Za-záéíóúýðþæöÁÉÍÓÚÝÐÞÆÖ\\d.µ]'
EOS_SYMBOL = '[.:?!;)(]'


class Tokenizer:

    def __init__(self):
        self.abbreviations = self.read_list(ABBR_FILE)
        self.abbreviations_non_ending = self.read_list(ABBR_NONENDING_FILE)

    @staticmethod
    def read_list(filename: str) -> list:
        with open(filename) as f:
            return f.read().splitlines()

    def detect_sentences(self, text: str) -> list:
        """
            Takes a cleaned text as input and returns a list of sentences
            as strings. A white space split on these strings gives a token list, where punctuation has been
            separated from word tokens, except from digits and abbreviations. However, a full stop at the
            end of a sentence is always separated from the last token, even if the last token is an
            abbreviation.
        """
        sentences = []
        tokens = text.split()
        tmp_str = ''
        last_token = ''
        # loop through all tokens in text and determine sentence boundaries, store tokens ending with '.' in the
        # last_token variable
        for token in tokens:
            if not token:
                continue
            tokenized = token
            if not re.fullmatch(ALPHABETIC, token):
                tokenized = self.process_special_characters(token.strip())
            tmp_str = self.check_last_token(sentences, tmp_str, last_token, tokenized)
            # keep tokens ending with '.' for the next iteration
            last_token = self.update_last_token(tokenized)
            if last_token:
                continue
            tmp_str = self.update_tmp_string(sentences, tmp_str, tokenized)

        self.finish_sentence(sentences, tmp_str, last_token)
        return sentences

    def finish_sentence(self, sentences: list, tmp_string: str, last_token: str) -> None:
        """ Check the content of 'tmp_str' and 'last_token' and finish the sentence contained in 'tmp_str'.
        'sentences' is the list of sentences already detected from the input text. After processing
        'tmp_str' and 'last_token' we create a new sentence string to add to 'sentences'
        """
        # we might still have a dangling last token
        if last_token:
            sent = self.ensure_full_stop(tmp_string, last_token)
            sentences.append(sent)
            tmp_string = ''

        # last token of text might not have ended with an EOS symbol, we still want to collect the last token
        # and return
        if tmp_string:
            last_sentence = tmp_string
            if not re.match('.*' + ALPHABETIC + '.*|.*\\d+.*', last_sentence):
                # we dont' want to add a sentence only constisting of symbols, do we?
                # rather add to last sentence, was probably a mistake to finish that one
                if sentences:
                    sent = sentences[-1]
                    sent += ' ' + last_sentence
                    sentences[-1] = sent
            # add a full stop ef the current sentence does not end with an EOS symbol
           # elif not re.match(EOS_SYMBOL, last_sentence.strip()[-1]):
           #     sentences.append(tmp_string.strip() + ' .')
            else:
                sentences.append(tmp_string.strip())

    def update_tmp_string(self, sentences: list, tmp_string: str, tokenized: str) -> str:
        """ Append 'tokenized' to 'tmp_string', check if 'tokenized' represents an end of a sentence,
        if yes, create a new sentence from tmp_string and add to sentences. Return tmp_string, that
        might have been reset to an empty string if we had a full sentence."""
        tmp_string += tokenized + ' '
        if self.is_EOS(tokenized):
            sentences.append(tmp_string)
            tmp_string = ''
        return tmp_string

    def update_last_token(self, tokenized: str) -> str:
        if self.ends_with_dot(tokenized):
            return tokenized
        return ''

    def check_last_token(self, sentences: list, tmp_string: str, last_token: str, tokenized: str) -> str:
        if last_token:
            if not self.is_full_stop_EOS(tokenized, last_token):
                tmp_string = self.append_token(tmp_string, last_token)
            else:
                sentence = self.ensure_full_stop(tmp_string, last_token)
                sentences.append(sentence)
                tmp_string = ''
        return tmp_string

    @staticmethod
    def append_token(tmp_string: str, token: str) -> str:
        """ 'token' might end with ' .', delete the space, because we are dealing with an abbreviation
        or a digits pattern that should not contain a space before the '.'"""
        token = token.replace(' ', '')
        return tmp_string + token + ' '

    @staticmethod
    def ensure_full_stop(tmp_string: str, token: str) -> str:
        """ Finish a sentence, take a look if the tmp_string content has a correct
        sentence ending, if not, add a ' .' to the sentence and return."""
        tmp_string += token.strip()
       # if not tmp_string.endswith(' .') and not tmp_string.endswith(' . \"'):
       #     tmp_string = tmp_string[:-1] + ' .'
        return tmp_string

    @staticmethod
    def ends_with_dot(token: str) -> bool:
        """ A token might end with a dot or a dot plus a quotation mark.
        TODO: we might have to add more patterns here. """
        return token.endswith('.') or token.endswith('. \"')

    def is_full_stop_EOS(self, current: str, last: str) -> bool:
        """ If last token ended with a dot we look at if the current token starts with an uppercase character and if
        last token is non sentence ending abbreviation. Generally, if next token starts with an uppercase letter
        we have an EOS unless the last token (the dot - token) is a one letter uppercase abbreviation or a defined
        non sentence ending abbreviation (like 'Hr.' which should always be followed by a name)
        """
        if not current:
            return False
        if current[0].isupper() or current[0] == '"':
            return not self.is_uppercase_abbr(last) and last.lower() not in self.abbreviations_non_ending
        return False

    @staticmethod
    def is_EOS(token: str) -> bool:
        """ Most EOS symbols are not as ambiguous like the dot, check for them here. The ':' is a matter of
        definition, we define it as EOS for now at least."""

        return token.endswith(' ?') or token.endswith('? "') or token.endswith(' !') or token.endswith('! "') \
               or token.endswith(' :') or token.endswith(' :)')

    def process_special_characters(self, token: str) -> str:
        """
        If a token contains some other character(s) than alphabetic characters, we take a closer look.
        """
        # First, check if we need to process the token, several categories do not need further processing even if
        # the token contains a non-alphabetic character, just return the token as is
        if not self.should_process(token):
            return token

        # For all kinds of punctuation we need to insert spaces at the correct positions
        # Patterns:
        processed_token = token
        # insert space after these symbols: '(', '[', '{'
        insert_space_after_anywhere = '([\\(\\[\\{])'
        # insert space before these symbols: ')', '[', '}' TODO: shouldn't '[' be ']' ?
        insert_space_before_anywhere = '([\\)\\]\\}])'
        # insert space after these symbols at the beginning of a token: '"',
        insert_space_after_if_beginning = '^(\")(.+)'
        # insert space before these symbols at the end of a token: '"', ':', ',', '.', '!', '?'
        insert_space_before_if_end = '(.+)([\":,.!?])$'
        # insert space before these symbols if two of them occur at the end of a token
        insert_space_before_if_end_and_punct = '(.+)([\":,.!?])(\\s[\":,.!?])$'

        # Replacements
        processed_token = re.sub(insert_space_after_anywhere, '\\g<1> ', processed_token)
        processed_token = re.sub(insert_space_before_anywhere, ' \\g<1>', processed_token)
        processed_token = re.sub(insert_space_after_if_beginning, '\\g<1> \\g<2>', processed_token)
        processed_token = re.sub(insert_space_before_if_end, '\\g<1> \\g<2>', processed_token)
        processed_token = re.sub(insert_space_before_if_end_and_punct, '\\g<1> \\g<2>\\g<3>', processed_token)

        return processed_token

    def should_process(self, token: str) -> bool:
        """ We assume 'token' contains some non-alphabetic characters and we test if it needs further processing.
        Tokens of size 0 or 1 do not need processing, and digits (with or without punctuation) and defined
        abbreviations are also not to be processed further. For all other tokens the method returns True
        """
        if len(token) <= 1:
            return False
        # a simple cardinal or ordinal number
        if re.fullmatch('\\d+\\.?', token):
            return False
        # a more complex combination of digits and punctuations, e.g. dates and large numbers
        if re.fullmatch('(\\d+[.,:]\\d+)+[,.]?', token):
            return False
        # telephone number or 'kennitala', don't split on hyphen
        if re.fullmatch('\\d{3}-\\d{4}', token):
            return False
        if re.fullmatch('\\d{6}-\\d{4}', token):
            return False
        # don't split on hyphen if we have a digits pattern with more than one hyphen
        if re.fullmatch('(\\d+-){2,}\\d+', token):
            return False
        # don't split on hyphen for non-digits
        if re.fullmatch('[^\\d]+-[^\\d]+', token):
            return False
        # don't split on slash for small number of chars on each side (digits or letters)
        if re.fullmatch(WORD_CHAR + '{1,3}/' + WORD_CHAR + '{1,3}', token):
            return False
        # special cases - resolve! We need to have simple rules for when to split and when not, the tokenizer
        # should not have to know too much about the normalizer!
        if re.match('(millj./)|(.+/klst)|(.+/kwst)|(.+/gwst)|(.+/gw\\.st)|(.+/mwst)|(.+/twst)|(.+/m²)|(.+/m³)'
                    '|(.+/mm²)|(.+/mm³)|(.+/cm²)|(.+/cm³)|(.+/ferm)|(.+/%)', token.lower()):
            return False
        # don't split smileys TODO: add more patterns here
        if re.fullmatch('(:\))|(:\()', token):
            return False
        # don't do anything with tokens that look like links and e-mail addresses
        if re.match('(www)|(http)|@', token):
            return False
        # don't process abbreviations
        if self.is_abbreviation(token):
            return False
        if self.is_uppercase_abbr(token):
            return False
        return True

    def is_uppercase_abbr(self, token: str) -> bool:
        return re.match('(' + UPPER_CASE + '\\.)+', token) and not self.is_abbreviation(token)

    def is_abbreviation(self, token: str) -> bool:
        if token.lower() in self.abbreviations:
            return True
        return token.lower() in self.abbreviations_non_ending


def main():
    input_text = 'hvort viltu fjórða, fimmta eða sjötta valkostinn?'
    tokenizer = Tokenizer()
    sentences = tokenizer.detect_sentences(input_text)
    for sent in sentences:
        print(sent)


if __name__ == '__main__':
    main()


from regina_normalizer import normalizer
from regina_normalizer import tokenizer
import pytest
import re


def normalize(text, domain):
    return normalizer.input_string(text, domain)


def test_one_card():
	# masculine singular
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 1 vini', 'other').strip()) == 'ég fékk gjöf frá einum vini'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 1 skóg?', 'other').strip()) == 'ætlarðu að höggva einn skóg ?'
    assert re.sub("\s+", " ", normalize('það var 1 stóll þarna', 'other').strip()) == 'það var einn stóll þarna'
    assert re.sub("\s+", " ", normalize('ætlarðu til 1 manns?', 'other').strip()) == 'ætlarðu til eins manns ?'
    # feminine singular
    assert normalize('1 kona', 'other').strip() == 'ein kona'
    assert re.sub("\s+", " ", normalize('það var talað um 1 konu', 'other').strip()) == 'það var talað um eina konu'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 1 vinkonu', 'other').strip()) == 'ég fékk gjöf frá einni vinkonu'
    assert re.sub("\s+", " ", normalize('ætlarðu til 1 konu?', 'other').strip()) == 'ætlarðu til einnar konu ?'
    # neutral singular
    assert normalize('1 borð', 'other').strip() == 'eitt borð'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 1 tré?', 'other').strip()) == 'ætlarðu að höggva eitt tré ?'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 1 barni', 'other').strip()) == 'ég fékk gjöf frá einu barni'
    assert re.sub("\s+", " ", normalize('ætlarðu til 1 lands?', 'other').strip()) == 'ætlarðu til eins lands ?'

def test_two_card():
	# masculine singular
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 2 vinum', 'other').strip()) == 'ég fékk gjöf frá tveimur vinum'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 2 skóga?', 'other').strip()) == 'ætlarðu að höggva tvo skóga ?'
    assert re.sub("\s+", " ", normalize('það voru 2 stólar þarna', 'other').strip()) == 'það voru tveir stólar þarna'
    assert re.sub("\s+", " ", normalize('ætlarðu til 2 manna?', 'other').strip()) == 'ætlarðu til tveggja manna ?'
    # feminine singular
    assert normalize('2 konur', 'other').strip() == 'tvær konur'
    assert re.sub("\s+", " ", normalize('það var talað um 2 konur', 'other').strip()) == 'það var talað um tvær konur'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 2 vinkonum', 'other').strip()) == 'ég fékk gjöf frá tveimur vinkonum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 2 kvenna?', 'other').strip()) == 'ætlarðu til tveggja kvenna ?'
    # neutral singular
    assert normalize('2 borð', 'other').strip() == 'tvö borð'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 2 tré?', 'other').strip()) == 'ætlarðu að höggva tvö tré ?'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 2 börnum', 'other').strip()) == 'ég fékk gjöf frá tveimur börnum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 2 landa?', 'other').strip()) == 'ætlarðu til tveggja landa ?'

def test_three_card():
	# masculine singular
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 3 vinum', 'other').strip()) == 'ég fékk gjöf frá þremur vinum'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 3 skóga?', 'other').strip()) == 'ætlarðu að höggva þrjá skóga ?'
    assert re.sub("\s+", " ", normalize('það voru 3 stólar þarna', 'other').strip()) == 'það voru þrír stólar þarna'
    assert re.sub("\s+", " ", normalize('ætlarðu til 3 manna?', 'other').strip()) == 'ætlarðu til þriggja manna ?'
    # feminine singular
    assert normalize('3 konur', 'other').strip() == 'þrjár konur'
    assert re.sub("\s+", " ", normalize('það var talað um 3 konur', 'other').strip()) == 'það var talað um þrjár konur'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 3 vinkonum', 'other').strip()) == 'ég fékk gjöf frá þremur vinkonum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 3 kvenna?', 'other').strip()) == 'ætlarðu til þriggja kvenna ?'
    # neutral singular
    assert normalize('3 borð', 'other').strip() == 'þrjú borð'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 3 tré?', 'other').strip()) == 'ætlarðu að höggva þrjú tré ?'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 3 börnum', 'other').strip()) == 'ég fékk gjöf frá þremur börnum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 3 landa?', 'other').strip()) == 'ætlarðu til þriggja landa ?'

def test_four_card():
	# masculine singular
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 4 vinum', 'other').strip()) == 'ég fékk gjöf frá fjórum vinum'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 4 skóga?', 'other').strip()) == 'ætlarðu að höggva fjóra skóga ?'
    assert re.sub("\s+", " ", normalize('það voru 4 stólar þarna', 'other').strip()) == 'það voru fjórir stólar þarna'
    assert re.sub("\s+", " ", normalize('ætlarðu til 4 manna?', 'other').strip()) == 'ætlarðu til fjögurra manna ?'
    # feminine singular
    assert normalize('4 konur', 'other').strip() == 'fjórar konur'
    assert re.sub("\s+", " ", normalize('það var talað um 4 konur', 'other').strip()) == 'það var talað um fjórar konur'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 4 vinkonum', 'other').strip()) == 'ég fékk gjöf frá fjórum vinkonum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 4 kvenna?', 'other').strip()) == 'ætlarðu til fjögurra kvenna ?'
    # neutral singular
    assert normalize('4 borð', 'other').strip() == 'fjögur borð'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 4 tré?', 'other').strip()) == 'ætlarðu að höggva fjögur tré ?'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 4 börnum', 'other').strip()) == 'ég fékk gjöf frá fjórum börnum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 4 landa?', 'other').strip()) == 'ætlarðu til fjögurra landa ?'

def test_no_context_card():
	# masculine singular
    assert re.sub("\s+", " ", normalize('hún var 1 þarna', 'other').strip()) == 'hún var eitt þarna'
    assert re.sub("\s+", " ", normalize('þau voru 2 þarna', 'other').strip()) == 'þau voru tvö þarna'
    assert re.sub("\s+", " ", normalize('þau voru 3 þarna', 'other').strip()) == 'þau voru þrjú þarna'
    assert re.sub("\s+", " ", normalize('þau voru 4 þarna', 'other').strip()) == 'þau voru fjögur þarna'

def test_other_ones_card():
	# masculine singular
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 5 vinum', 'other').strip()) == 'ég fékk gjöf frá fimm vinum'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 6 skóga?', 'other').strip()) == 'ætlarðu að höggva sex skóga ?'
    assert re.sub("\s+", " ", normalize('það voru 7 stólar þarna', 'other').strip()) == 'það voru sjö stólar þarna'
    assert re.sub("\s+", " ", normalize('ætlarðu til 8 manna?', 'other').strip()) == 'ætlarðu til átta manna ?'
    # feminine singular
    assert normalize('9 konur', 'other').strip() == 'níu konur'
    assert re.sub("\s+", " ", normalize('það var talað um 10 konur', 'other').strip()) == 'það var talað um tíu konur'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 11 vinkonum', 'other').strip()) == 'ég fékk gjöf frá ellefu vinkonum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 12 kvenna?', 'other').strip()) == 'ætlarðu til tólf kvenna ?'
    # neutral singular
    assert normalize('13 borð', 'other').strip() == 'þrettán borð'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 14 tré?', 'other').strip()) == 'ætlarðu að höggva fjórtán tré ?'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 15 börnum', 'other').strip()) == 'ég fékk gjöf frá fimmtán börnum'
    assert re.sub("\s+", " ", normalize('ætlarðu til 16 landa?', 'other').strip()) == 'ætlarðu til sextán landa ?'

    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 17 vinum', 'other').strip()) == 'ég fékk gjöf frá sautján vinum'
    assert re.sub("\s+", " ", normalize('ætlarðu að höggva 18 skóga?', 'other').strip()) == 'ætlarðu að höggva átján skóga ?'
    assert re.sub("\s+", " ", normalize('það voru 19 stólar þarna', 'other').strip()) == 'það voru nítján stólar þarna'

def test_onehundred_card():
    assert re.sub("\s+", " ", normalize('ég á 0 vini', 'other').strip()) == 'ég á núll vini'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 2101 vini', 'other').strip()) == 'ég fékk gjöf frá tvö þúsund eitt hundrað og einum vini'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 2120. vininum', 'other').strip()) == 'ég fékk gjöf frá tvö þúsund eitt hundrað og tuttugasta vininum'
    assert re.sub("\s+", " ", normalize('þetta voru 104 konur', 'other').strip()) == 'þetta voru hundrað og fjórar konur'
    assert re.sub("\s+", " ", normalize('þetta var 120. konan', 'other').strip()) == 'þetta var hundrað og tuttugasta konan'
    assert re.sub("\s+", " ", normalize('þetta var 3121. konan', 'other').strip()) == 'þetta var þrjú þúsund eitt hundrað tuttugasta og fyrsta konan'
    assert re.sub("\s+", " ", normalize('næst eru 100,6 grömm', 'other').strip()) == 'næst eru hundrað komma sex grömm'
    assert re.sub("\s+", " ", normalize('næst er 121. hálfleikur', 'other').strip()) == 'næst er hundrað tuttugasti og fyrsti hálfleikur'
    # if we write "21004" then it will say "tveir einn núll núll fjórir", it has to have a thousands separator for it to expand correctly
    assert re.sub("\s+", " ", normalize('21.004 menn', 'other').strip()) == 'tuttugu og eitt þúsund og fjórir menn'
    assert re.sub("\s+", " ", normalize('21.020. mennirnir', 'other').strip()) == 'tuttugu og eitt þúsund og tuttugustu mennirnir'
    assert re.sub("\s+", " ", normalize('21.304 menn', 'other').strip()) == 'tuttugu og eitt þúsund þrjú hundruð og fjórir menn'
    assert re.sub("\s+", " ", normalize('1000 kr.', 'other').strip()) == 'þúsund krónur .'

def test_dozens_card():
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 21 vini', 'other').strip()) == 'ég fékk gjöf frá tuttugu og einum vini'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 32 vinum', 'other').strip()) == 'ég fékk gjöf frá þrjátíu og tveimur vinum'
    assert re.sub("\s+", " ", normalize('þetta voru 43 konur', 'other').strip()) == 'þetta voru fjörutíu og þrjár konur'
    assert re.sub("\s+", " ", normalize('þetta voru 54 konur', 'other').strip()) == 'þetta voru fimmtíu og fjórar konur'
    assert re.sub("\s+", " ", normalize('þetta voru 64 börn', 'other').strip()) == 'þetta voru sextíu og fjögur börn'
    assert re.sub("\s+", " ", normalize('næst eru 70,4 grömm', 'other').strip()) == 'næst eru sjötíu komma fjögur grömm'
    assert re.sub("\s+", " ", normalize('næst eru 85,2 kg', 'other').strip()) == 'næst eru áttatíu og fimm komma tvö kíló'
    assert re.sub("\s+", " ", normalize('95 menn', 'other').strip()) == 'níutíu og fimm menn'

def test_hundreds_thousands_card():
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 211 vinum', 'other').strip()) == 'ég fékk gjöf frá tvö hundruð og ellefu vinum'
    assert re.sub("\s+", " ", normalize('þetta var 330. vinurinn', 'other').strip()) == 'þetta var þrjú hundruð og þrítugasti vinurinn'
    assert re.sub("\s+", " ", normalize('þetta voru 400,3 konur', 'other').strip()) == 'þetta voru fjögur hundruð komma þrjár konur'
    assert re.sub("\s+", " ", normalize('þetta voru 2554 konur', 'other').strip()) == 'þetta voru tvö þúsund fimm hundruð fimmtíu og fjórar konur'
    assert re.sub("\s+", " ", normalize('þetta var 3121. konan', 'other').strip()) == 'þetta var þrjú þúsund eitt hundrað tuttugasta og fyrsta konan'
    assert re.sub("\s+", " ", normalize('4005 börn týndust', 'other').strip()) == 'fjögur þúsund og fimm börn týndust'
    assert re.sub("\s+", " ", normalize('5005. barnið týndist', 'other').strip()) == 'fimm þúsundasta og fimmta barnið týndist'
    assert re.sub("\s+", " ", normalize('6022 börn fundust', 'other').strip()) == 'sex þúsund tuttugu og tvö börn fundust'

def test_tens_card():
    assert re.sub("\s+", " ", normalize('1106 vinir', 'other').strip()) == 'ellefu hundruð og sex vinir'
    assert re.sub("\s+", " ", normalize('1200,5 börn', 'other').strip()) == 'tólf hundruð komma fimm börn'
    assert re.sub("\s+", " ", normalize('1532 konur', 'other').strip()) == 'fimmtán hundruð þrjátíu og tvær konur'
    assert re.sub("\s+", " ", normalize('16.124 ungmenni', 'other').strip()) == 'sextán þúsund eitt hundrað tuttugu og fjögur ungmenni'
    assert re.sub("\s+", " ", normalize('1800 kr.', 'other').strip()) == 'átján hundruð krónur .'
    assert re.sub("\s+", " ", normalize('árið var 1923', 'other').strip()) == 'árið var nítján hundruð tuttugu og þrjú'
    assert re.sub("\s+", " ", normalize('árið var 1922', 'other').strip()) == 'árið var nítján hundruð tuttugu og tvö'
    assert re.sub("\s+", " ", normalize('árið var 1903', 'other').strip()) == 'árið var nítján hundruð og þrjú'
    assert re.sub("\s+", " ", normalize('árið var 1874', 'other').strip()) == 'árið var átján hundruð sjötíu og fjögur'
    assert re.sub("\s+", " ", normalize('árið var 1873', 'other').strip()) == 'árið var átján hundruð sjötíu og þrjú'
    assert re.sub("\s+", " ", normalize('11.005 menn', 'other').strip()) == 'ellefu þúsund og fimm menn'
    assert re.sub("\s+", " ", normalize('14.020. maðurinn', 'other').strip()) == 'fjórtán þúsund og tuttugasti maðurinn'
    assert re.sub("\s+", " ", normalize('17.405 menn', 'other').strip()) == 'sautján þúsund fjögur hundruð og fimm menn'

def test_million_card():
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 10500 vinum', 'other').strip()) == 'ég fékk gjöf frá tíu þúsund og fimm hundruð vinum'
    assert re.sub("\s+", " ", normalize('ég fékk gjöf frá 12020. vininum', 'other').strip()) == 'ég fékk gjöf frá tólf þúsund og tuttugasta vininum'
    assert re.sub("\s+", " ", normalize('þetta voru 30104 konur', 'other').strip()) == 'þetta voru þrjátíu þúsund eitt hundrað og fjórar konur'
    assert re.sub("\s+", " ", normalize('þetta voru 2.100.020 konur', 'other').strip()) == 'þetta voru tvær milljónir eitt hundrað þúsund og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 100.020 konur', 'other').strip()) == 'þetta voru hundrað þúsund og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 2.100.020. konurnar', 'other').strip()) == 'þetta voru tvær milljónir eitt hundrað þúsund og tuttugustu konurnar'
    assert re.sub("\s+", " ", normalize('þetta voru 100.020. konurnar', 'other').strip()) == 'þetta voru hundrað þúsund og tuttugustu konurnar'
    assert re.sub("\s+", " ", normalize('þetta voru 2.100.020. konurnar', 'other').strip()) == 'þetta voru tvær milljónir eitt hundrað þúsund og tuttugustu konurnar'
    assert re.sub("\s+", " ", normalize('þetta voru 100.020. konurnar', 'other').strip()) == 'þetta voru hundrað þúsund og tuttugustu konurnar'
    assert re.sub("\s+", " ", normalize('þetta voru 2.130.020 konur', 'other').strip()) == 'þetta voru tvær milljónir eitt hundrað og þrjátíu þúsund og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 130.020 konur', 'other').strip()) == 'þetta voru hundrað og þrjátíu þúsund og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 2.135.020 konur', 'other').strip()) == 'þetta voru tvær milljónir eitt hundrað þrjátíu og fimm þúsund og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 135.020 konur', 'other').strip()) == 'þetta voru hundrað þrjátíu og fimm þúsund og tuttugu konur'


def test_dozen_million_card():
    assert re.sub("\s+", " ", normalize('þetta voru 14002 konur', 'other').strip()) == 'þetta voru fjórtán þúsund og tvær konur'
    assert re.sub("\s+", " ", normalize('þetta var 20020. konan', 'other').strip()) == 'þetta var tuttugu þúsund og tuttugasta konan'
    assert re.sub("\s+", " ", normalize('þetta voru 40.520 konur', 'other').strip()) == 'þetta voru fjörutíu þúsund fimm hundruð og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 45.425 konur', 'other').strip()) == 'þetta voru fjörutíu og fimm þúsund fjögur hundruð tuttugu og fimm konur'

def test_hundreds_million_card():
    assert re.sub("\s+", " ", normalize('þetta voru 200.520 konur', 'other').strip()) == 'þetta voru tvö hundruð þúsund fimm hundruð og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 200.020 konur', 'other').strip()) == 'þetta voru tvö hundruð þúsund og tuttugu konur'
    assert re.sub("\s+", " ", normalize('þetta voru 220000 konur', 'other').strip()) == 'þetta voru tvö hundruð og tuttugu þúsund konur'
    assert re.sub("\s+", " ", normalize('þetta voru 225.532 konur', 'other').strip()) == 'þetta voru tvö hundruð tuttugu og fimm þúsund fimm hundruð þrjátíu og tvær konur'


def test_cardinal_big():
    assert normalize('100.321.324.123 borð', 'other').strip() == 'hundrað milljarðar þrjú hundruð tuttugu og ein milljón þrjú hundruð tuttugu og fjögur þúsund eitt hundrað tuttugu og þrjú borð'
    assert normalize('321.324.123 borð', 'other').strip() == 'þrjú hundruð tuttugu og ein milljón þrjú hundruð tuttugu og fjögur þúsund eitt hundrað tuttugu og þrjú borð'

def test_neutral_card():
    assert re.sub("\s+", " ", normalize('það voru 22 konur þarna', 'other').strip()) == 'það voru tuttugu og tvær konur þarna'
    assert re.sub("\s+", " ", normalize('það voru 1922 menn þarna', 'other').strip()) == 'það voru nítján hundruð tuttugu og tveir menn þarna'
    assert re.sub("\s+", " ", normalize('það voru 1873 börn þarna', 'other').strip()) == 'það voru átján hundruð sjötíu og þrjú börn þarna'
    assert re.sub("\s+", " ", normalize('árið var 1903', 'other').strip()) == 'árið var nítján hundruð og þrjú'
    assert re.sub("\s+", " ", normalize('árið var 1912', 'other').strip()) == 'árið var nítján hundruð og tólf'
    assert re.sub("\s+", " ", normalize('árið var 1922', 'other').strip()) == 'árið var nítján hundruð tuttugu og tvö'
    assert re.sub("\s+", " ", normalize('árið var 1920', 'other').strip()) == 'árið var nítján hundruð og tuttugu'
    assert re.sub("\s+", " ", normalize('árið var 1927', 'other').strip()) == 'árið var nítján hundruð tuttugu og sjö'
    assert re.sub("\s+", " ", normalize('árið var 1722', 'other').strip()) == 'árið var sautján hundruð tuttugu og tvö'
    assert re.sub("\s+", " ", normalize('árið var 1933', 'other').strip()) == 'árið var nítján hundruð þrjátíu og þrjú'
    assert re.sub("\s+", " ", normalize('árið var 1942', 'other').strip()) == 'árið var nítján hundruð fjörutíu og tvö'
    assert re.sub("\s+", " ", normalize('árið var 1951', 'other').strip()) == 'árið var nítján hundruð fimmtíu og eitt'
    assert re.sub("\s+", " ", normalize('árið var 1874', 'other').strip()) == 'árið var átján hundruð sjötíu og fjögur'
    assert re.sub("\s+", " ", normalize('árið var 1989', 'other').strip()) == 'árið var nítján hundruð áttatíu og níu'
    assert re.sub("\s+", " ", normalize('barnið var 3 ára', 'other').strip()) == 'barnið var þriggja ára'
    assert re.sub("\s+", " ", normalize('árið var 3 þegar ég var 4 ára', 'other').strip()) == 'árið var þrjú þegar ég var fjögurra ára'
    assert re.sub("\s+", " ", normalize('árið var 22', 'other').strip()) == 'árið var tuttugu og tvö'
    assert re.sub("\s+", " ", normalize('árið var 11.722', 'other').strip()) == 'árið var ellefu þúsund sjö hundruð tuttugu og tvö'
    assert re.sub("\s+", " ", normalize('árið var 933', 'other').strip()) == 'árið var níu hundruð þrjátíu og þrjú'
    assert re.sub("\s+", " ", normalize('árið var 42 en ég var ekki fædd', 'other').strip()) == 'árið var fjörutíu og tvö en ég var ekki fædd'
    assert re.sub("\s+", " ", normalize('árið var 251', 'other').strip()) == 'árið var tvö hundruð fimmtíu og eitt'
    assert re.sub("\s+", " ", normalize('árið var 174', 'other').strip()) == 'árið var hundrað sjötíu og fjögur'




from . import number_patterns as npa
from . import tuple_rules as tp

decimal_thousands_tuples = [
((npa.zeropnt_ptrn + "\d{9}0\d*$"), '.*', 'point10', ' núll'),
((npa.zeropnt_ptrn + "\d{8}0\d*$"), '.*', 'point9', ' núll'),
((npa.zeropnt_ptrn + "\d{7}0\d*$"), '.*', 'point8', ' núll'),
((npa.zeropnt_ptrn + "\d{6}0\d*$"), '.*', 'point7', ' núll'),
((npa.zeropnt_ptrn + "\d{5}0\d*$"), '.*', 'point6', ' núll'),
((npa.zeropnt_ptrn + "\d{4}0\d*$"), '.*', 'point5', ' núll'),
((npa.zeropnt_ptrn + "\d{3}0\d*$"), '.*', 'point4', ' núll'),
((npa.zeropnt_ptrn + "\d{2}0\d*$"), '.*', 'point3', ' núll'),
((npa.zeropnt_ptrn + "\d0\d*$"), '.*', 'point2', ' núll'),
((npa.zeropnt_ptrn + "0\d*$"), '.*', 'points', ' komma núll'),
("^0,\d+$", '.*', 'ones', ' núll')]


for rule, string, number in tp.ones_zip:
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + number + "\d*$"), rule, 'points', ' komma' + string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d" + number + "\d*$"), rule, 'point2', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{2}" + number + "\d*$"), rule, 'point3', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{3}" + number + "\d*$"), rule, 'point4', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{4}" + number + "\d*$"), rule, 'point5', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{5}" + number + "\d*$"), rule, 'point6', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{6}" + number + "\d*$"), rule, 'point7', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{7}" + number + "\d*$"), rule, 'point8', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{8}" + number + "\d*$"), rule, 'point9', string))
    decimal_thousands_tuples.append(((npa.zeropnt_ptrn + "\d{9}" + number + "\d*$"), rule, 'point10', string))

#for rule, string, number in tp.dec_ones_male:
#    decimal_thousands_tuples.append(((npa.ones_ptrn_no11 + number + npa.dec_ptrn_def), rule, 'ones', string))


from setuptools import setup, find_packages

with open("README.md", "r") as readme:
    long_description = readme.read()

setup(
	name="regina-normalizer",
	version="0.0.6",
	author = "Helga Svala Sigurðardóttir, Anna Björk Nikulásdóttir",
	author_email = "helgas@ru.is",
	url="https://github.com/cadia-lvl/regina-normalizer/tree/master/regina_package",
	description = "A text normalization package for Icelandic",
	license="MIT",
	long_description = long_description,
	long_description_content_type = "text/markdown",
	packages=find_packages(),

	package_data={"": ["*.txt"]},
	include_package_data=True,
	install_requires=['setuptools', 'wandb'],
	python_requires='>=3.5',
	entry_points={
			'console_scripts': [
				'regina_normalizer=regina_normalizer.main:main'
							   ]
				 }
	)




from . import number_patterns as npa
from . import tuple_rules as tp

ordinal_million_tuples = []

for rule, letter in tp.dozens_ordinal_letters:
    ordinal_million_tuples.append((npa.tns_ptrn + "10\.?000\.$", rule, 'ten thousands', ' tíu þúsund' + letter))
    ordinal_million_tuples.append((npa.hndrds_ptrn + "100\.?0([01][1-9]|10)\.$", rule, 'hundred thousands', ' eitt hundrað þúsund' + letter + ' og'))
    ordinal_million_tuples.append(("^100\.?0([01][1-9]|10)\.$", rule, 'hundred thousands', 'hundrað þúsund' + letter + ' og'))        
    ordinal_million_tuples.append((npa.hndrds_ptrn + "100\.?000\.$", rule, 'hundred thousands',' eitt hundrað þúsund' + letter))
    ordinal_million_tuples.append(("^100\.?000\.$", rule, 'hundred thousands', 'hundrað þúsund' + letter))
    for string, number in tp.dozens_zip:
        ordinal_million_tuples.append((npa.tns_ptrn + number + "0\.?0([01][1-9]|10)\.$", rule, 'ten thousands', string + 'hæjó þúsund' + letter + ' og'))
        ordinal_million_tuples.append((npa.tns_ptrn + number + "0\.?000\.$", rule, 'ten thousands', string + ' þúsund' + letter))
    for string, number in tp.hundreds_thousands_zip:
        ordinal_million_tuples.append((npa.hndrds_ptrn + number + "00\.?0([01][1-9]|10)\.$", rule, 'hundred thousands', string + ' hundruðþúsund' + letter + ' og'))
        ordinal_million_tuples.append((npa.hndrds_ptrn + number + "00\.?000\.$", rule, 'hundred thousands',string + ' hundruðþúsund' + letter))



from . import tuple_rules as tp
from . import number_patterns as npa

cardinal_thousands_tuples = [(npa.hndrds_ptrn_def + "1([01][1-9]|[1-9]0)" + npa.dec_ptrn, '.*', 'hundreds', ' eitt hundrað og'),
                            (npa.hndrds_ptrn_def + "1[2-9]0\.$", '.*', 'hundreds', ' eitt hundrað og'),
                            ("^1([01][1-9]|[1-9]0)" + npa.dec_ptrn, '.*', 'hundreds', 'hundrað og'),
                            ("^1([2-9]0)\.$", '.*', 'hundreds', 'hundrað og'),
                            (npa.hndrds_ptrn_def + "1([2-9][1-9]|00)" + npa.dec_ptrn_ordinal, '.*', 'hundreds', ' eitt hundrað'),
                            ("^100" + npa.dec_ptrn, '.*', 'hundreds', 'hundrað'),
                            ("^1([2-9][1-9])" + npa.dec_ptrn_ordinal, '.*', 'hundreds', 'hundrað'),
                            (npa.ones_ptrn_no11 + "1" + npa.thsnds_and_ptrn_cardinal + npa.dec_ptrn, '.*', 'thousands', ' eitt þúsund og'),
                            (npa.ones_ptrn_no11 + "1" + npa.thsnds_and_ptrn_ordinal, '.*', 'thousands', ' eitt þúsund og'),
                            (npa.ones_ptrn_no11 + "1\.?((000$)|(([1-9](?!00)\d{2})|(0[2-9][1-9]))" + npa.dec_ptrn_ordinal + ")", '.*', 'thousands', ' eitt þúsund'),
                            ("^1\.?000$", '.*', 'thousands', ' þúsund')]

for string, number in tp.dozens_zip:
    cardinal_thousands_tuples.append((npa.tns_ptrn + number + "0" + npa.dec_ptrn, '.*', 'dozens', string))
    cardinal_thousands_tuples.append((npa.tns_ptrn + number + "[1-9]" + npa.dec_ptrn, '.*', 'dozens', string + ' og'))
    
for string, number in tp.hundreds_thousands_zip:
    cardinal_thousands_tuples.append((npa.hndrds_ptrn + number + "([01][1-9]|[1-9]0)" + npa.dec_ptrn, '.*', 'hundreds', string + ' hundruð og'))
    cardinal_thousands_tuples.append((npa.hndrds_ptrn + number + "[2-9]0\.$", '.*', 'hundreds', string + ' hundruð og'))
    cardinal_thousands_tuples.append((npa.hndrds_ptrn + number + "00" + npa.dec_ptrn, '.*', 'hundreds', string + ' hundruð'))
    cardinal_thousands_tuples.append((npa.hndrds_ptrn + number + "[2-9][1-9]" + npa.dec_ptrn_ordinal, '.*', 'hundreds', string + ' hundruð'))

    cardinal_thousands_tuples.append((npa.ones_ptrn_no11 + number + npa.thsnds_and_ptrn_cardinal + npa.dec_ptrn, '.*', 'thousands', string + ' þúsund og'))
    cardinal_thousands_tuples.append((npa.ones_ptrn_no11 + number + npa.thsnds_and_ptrn_ordinal, '.*', 'thousands', string + ' þúsund og'))
    cardinal_thousands_tuples.append((npa.ones_ptrn_no11 + number + "\.?((000$)|(([1-9](?!00)\d{2})|(0[2-9][1-9]))" + npa.dec_ptrn_ordinal + ")", '.*', 'thousands', string + ' þúsund'))
    
for string, number in tp.tens_zip[1:]:
    cardinal_thousands_tuples.append(("^" + number + "([01][1-9]|[1-9]0)" + npa.dec_ptrn, '.*', 'hundreds', string + ' hundruð og'))
    cardinal_thousands_tuples.append(("^" + number + "00" + npa.dec_ptrn, '.*', 'hundreds', string + ' hundruð'))
    cardinal_thousands_tuples.append(("^" + number + "([2-9][1-9])" + npa.dec_ptrn_ordinal, '.*', 'hundreds', string + ' hundruð'))
    cardinal_thousands_tuples.append(("^" + number + "([01][1-9]|[1-9]0)" + npa.dec_ptrn, '.*', 'thousands', ""))
    cardinal_thousands_tuples.append(("^" + number + "00" + npa.dec_ptrn, '.*', 'thousands', ""))
    cardinal_thousands_tuples.append(("^" + number + "([2-9][1-9])" + npa.dec_ptrn_ordinal, '.*', 'thousands', ""))
for string, number in tp.tens_zip:
    cardinal_thousands_tuples.append((npa.tns_ptrn + number + npa.thsnds_and_ptrn_cardinal + npa.dec_ptrn, '.*', 'thousands', string + ' þúsund og'))
    cardinal_thousands_tuples.append((npa.tns_ptrn + number + npa.thsnds_and_ptrn_ordinal, '.*', 'thousands', string + ' þúsund og'))
    cardinal_thousands_tuples.append((npa.tns_ptrn + number + npa.thsnds_ptrn_after + npa.dec_ptrn_ordinal, '.*', 'thousands', string + ' þúsund'))



# abn pkg
#!/usr/bin/env python3

import re
import sys
import time
from regina_normalizer import tokenizer
from regina_normalizer import abbr_functions as af
from regina_normalizer import number_functions as nf

def input_string(text_string, domain):
    tok = tokenizer.Tokenizer()
    sentences = tok.detect_sentences(text_string)
    normalized = []
    for sent in sentences:
        abbr_sent = af.replace_abbreviations(sent, domain)
        normalized.append(nf.handle_sentence(abbr_sent, domain))
    final_string = ' '.join(normalized)
    #print(final_string)
    return final_string

def input_file(input_file, output_file, domain):
    with open(input_file, 'r') as file:
        lines = file.readlines()
        lines = [line.rstrip() for line in lines]
    tok = tokenizer.Tokenizer()
    sentences = tok.detect_sentences(' '.join(lines))
    normalized = []
    for sent in sentences:
        abbr_sent = af.replace_abbreviations(sent, domain)
        normalized.append(nf.handle_sentence(abbr_sent, domain))
    with open(output_file, 'w') as f:
        for item in normalized:
            f.write("%s\n" % item)   
	
if __name__ == '__main__':
	if len(sys.argv) == 3:
		handle_input_command(sys.argv[1], sys.argv[2])
	elif len(sys.argv) == 4:
		handle_input_file(sys.argv[1], sys.argv[2], sys.argv[3])
		print("Done writing output file!")
	else:
		print("\nERROR MESSAGE\n\nFor command line, you need to write: python3 -W ignore main.py {string-to-be-normalized} {domain}\n"+
			  "For a text file, you need to write: python3 -W ignore main.py {file-to-be-normalized} {output-file-normalized} {domain}\n"+
			  "{domain} can be 'sport' or 'other'\n")


import re
import json
import os

from . import abbr_dict as abd 
from . import area_dict as ad
from . import currency_dict as cd
from . import denominator_dict as dend
from . import direction_dict as dird
from . import distance_dict as dd
from . import electronic_dict as ed
from . import period_dict as pd
from . import rest_dict as rd
from . import time_dict as td
from . import volume_dict as vd
from . import weight_dict as wd
from . import pre_help_dicts as phd
from . import symbols_dict as sd

direction_ptrn = "[SN]?V|N|[SN]?A|S"

pre_help_dict = phd.pre_help_dicts
abbr_dict = abd.abbr_dict
area_dict = ad.make_area_dict()
currency_dict = cd.make_currency_dict()
denominator_dict = dend.denominator_dict
direction_dict = dird.direction_dict
distance_dict = dd.make_distance_dict()
electronic_dict = ed.make_electronic_dict()
period_dict = pd.make_period_dict()
rest_dict = rd.make_rest_measure_dict()
time_dict = td.make_time_dict()
volume_dict = vd.make_volume_dict()
weight_dict = wd.make_weight_dict()
symb_dict = sd.symb_dict

# replace words according to the appropriate dictionary 
def replace_all(text, dic, ptrn=""):
    if re.findall(ptrn, text):
        for i, j in dic.items():
            text = re.sub(i, j, text)
    return text

def replace_abbreviations(sent, domain):
    sent = replace_all(sent, direction_dict, direction_ptrn)
    sent = replace_all(sent, pre_help_dict)
    sent = replace_all(sent, denominator_dict, "\/")
    sent = replace_all(sent, weight_dict, wd.weight_ptrn)
    sent = replace_all(sent, distance_dict, dd.distance_ptrn)
    sent = replace_all(sent, area_dict, ad.area_ptrn)
    sent = replace_all(sent, volume_dict, vd.volume_ptrn)
    sent = replace_all(sent, time_dict, td.time_ptrn)
    sent = replace_all(sent, currency_dict, cd.currency_ptrn)
    sent = replace_all(sent, electronic_dict, ed.electronic_ptrn)
    sent = replace_all(sent, rest_dict, rd.rest_ptrn) 
    sent = replace_all(sent, period_dict, pd.period_ptrn)
    sent = replace_all(sent, abbr_dict)
    if domain == 'other':
        sent = re.sub('(\W|^)(\d{1,3})(\/)(0|\d{1}\d+)(\W|$)', '\g<1>\g<2> \g<3> \g<4>\g<5>', sent)
        sent = re.sub('(\W|^)(\d{3}\d+)(\/)(\d+)(\W|$)', '\g<1>\g<2> \g<3> \g<4>\g<5>', sent)
    if domain == 'sport':
        sent = re.sub('(W|^)(\d{2}\d+)(\/)(\d+)(\W|$)', '\g<1>\g<2> \g<3> \g<4>\g<5>', sent)
        sent = re.sub('(W|^)(\d+)(\/)(\d{2}\d+)(\W|$)', '\g<1>\g<2> \g<3> \g<4>\g<5>', sent)
    return sent





# These control the case of the succeeding number, whether it's accusative, dative, genitive or a combination.
# Nominative is default
acc_words = '[Uu]m(fram|hverfis)|UM(FRAM|HVERFIS)|[Uu]m|UM|[Gg]egnum|GEGNUM|[Kk]ringum|KRINGUM|[Vv]ið|VIÐ|[Íí]|[Áá]'
dat_words = '[Ff]rá|FRÁ|[Aa][ðf]|A[ÐF]|[Áá]samt|ÁSAMT|[Gg]agnvart|GAGNVART|[Gg]egnt?|GEGNT?|[Hh]anda|HANDA|[Hh]já|HJÁ|[Mm]eð( )?fram|MEÐ( )?FRAM|[Mm]óti?|MÓTI?|[Uu]ndan|UNDAN|[Nn]álægt|NÁLÆGT'
gen_words = '[Tt]il|TIL|[Aa]uk|AUK|[Áá]n|ÁN|[Hh]andan|HANDAN|[Ii]nnan|INNAN|[Mm]eðal|MEÐAL|[Mm]egin|MEGIN|[Mm]ill(i|um)|MILL(I|UM)|[Oo]fan|OFAN|[Ss]akir|SAKIR|[Ss]ökum|SÖKUM|[Uu]tan|UTAN|[Vv]egna|VEGNA'
accdat_words = "[Ee]ftir|EFTIR|[Ff]yrir|FYRIR|[Mm]eð|MEÐ|[Uu]ndir|UNDIR|[Vv]ið|VIÐ|[Yy]fir|YFIR"
accgen_words = acc_words + "|" + gen_words
accdat_words_comb = acc_words + "|" + dat_words + "|" + accdat_words
accdatgen_words_comb = accdat_words_comb + "|" + gen_words

amounts = "([Hh]undr[au]ð|HUNDR[AU]Ð|[Þþ]úsund|ÞÚSUND|[Mm]illjón(ir)?|MILLJÓN(IR)?)"
# Dates can have a date consisting of a day from 01. to 31., a month, and a year of two to four digits
date_ptrn = r"^((([012]?[1-9]|3[01])\. ?)?(jan(úar)?|feb(rúar)?|mars?|apr(íl)?|maí|jú[nl]í?|ág(úst)?|sep(t(ember)?)?|okt(óber)?|nóv(ember)?|des(ember)?) )\d{2,4}$"
# The link patterns handle external patterns like https://mbl.is/innlent, internal patterns like https://localholst:8888, 
# mail patterns like name@address.com, twitter handles like @handle and hashtags, e.g. #thisrules2021
link_ptrn_external = "(https?:\/\/)?(www\.)?([A-ZÁÐÉÍÓÚÝÞÆÖa-záðéíóúýþæö\d\-_\.\/]+)?\.[A-ZÁÐÉÍÓÚÝÞÆÖa-záðéíóúýþæö\d\-_\.\/]+"
link_ptrn_internal = "(file|(https?:\/\/)?localhost):[A-ZÁÐÉÍÓÚÝÞÆÖa-záðéíóúýþæö\d_\?\/\.=\-\&\%\#]+"
link_ptrn_mail = "[A-ZÁÐÉÍÓÚÝÞÆÖa-záðéíóúýþæö\d\-_\.]*@[A-ZÁÐÉÍÓÚÝÞÆÖa-záðéíóúýþæö\d\-_\.]+(\.[A-Za-z])?"
link_ptrn_hashtag = "# ?[A-ZÁÐÉÍÓÚÝÞÆÖa-záðéíóúýþæö\d\-_]+"
link_ptrn_all = "^(" + link_ptrn_external + "|" + link_ptrn_internal + "|" + link_ptrn_mail + "|" + link_ptrn_hashtag + ")$"





from . import number_patterns as npa
from . import tuple_rules as tp

cardinal_million_tuples = [#(npa.tns_ptrn + "10" + npa.thsnds_and_ptrn_ordinal, '.*', 'ten thousands', ' tíu þúsund og'),
                            #(npa.tns_ptrn + "10" + npa.thsnds_ptrn_after + npa.dec_ptrn_ordinal, '.*', 'ten thousands', ' tíu þúsund'),
                            (npa.hndrds_ptrn + "100" + npa.thsnds_and_ptrn_cardinal + npa.dec_ptrn, '.*', 'hundred thousands', ' eitt hundrað þúsund og'),
                            ("^100" + npa.thsnds_and_ptrn_cardinal + npa.dec_ptrn, '.*', 'hundred thousands', 'hundrað þúsund og'),
                            (npa.hndrds_ptrn + "100" + npa.thsnds_and_ptrn_ordinal, '.*', 'hundred thousands', ' eitt hundrað þúsund og'),
                            ("^100" + npa.thsnds_and_ptrn_ordinal, '.*', 'hundred thousands', 'hundrað þúsund og'),

                            (npa.hndrds_ptrn_def + "100" + npa.thsnds_ptrn_after + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', ' eitt hundrað þúsund'),
                            ("^100" + npa.thsnds_ptrn_after + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', 'hundrað þúsund'),
                            (npa.hndrds_ptrn + "1" + npa.hndrd_and_thsnd + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', ' eitt hundrað og'),
                            ("^1" + npa.hndrd_and_thsnd + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', ' hundrað og'),
                            (npa.hndrds_ptrn + "1" + npa.hndrd_thsnd_after + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', ' eitt hundrað'),
                            ("^1" + npa.hndrd_thsnd_after + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', 'hundrað'),]

for string, number in tp.dozens_zip:
    cardinal_million_tuples.append((npa.tns_ptrn + number + "0" + npa.thsnds_and_ptrn_cardinal + npa.dec_ptrn, '.*', 'ten thousands', string + ' þúsund og'))
    cardinal_million_tuples.append((npa.tns_ptrn + number + "0" + npa.thsnds_and_ptrn_ordinal, '.*', 'ten thousands', string + ' þúsund og'))
    cardinal_million_tuples.append((npa.tns_ptrn + number + "0" + npa.thsnds_ptrn_after + npa.dec_ptrn_ordinal, '.*', 'ten thousands', string + ' þúsund'))
    cardinal_million_tuples.append((npa.tns_ptrn + number + "[1-9]\.?\d{3}" + npa.dec_ptrn_ordinal, '.*', 'ten thousands', string + ' og'))

for string, number in tp.hundreds_thousands_zip:
    cardinal_million_tuples.append((npa.hndrds_ptrn + number + "00" + npa.thsnds_ptrn_after + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', string + ' hundruð þúsund'))
    cardinal_million_tuples.append((npa.hndrds_ptrn + number + "00" + npa.thsnds_and_ptrn_cardinal + npa.dec_ptrn, '.*', 'hundred thousands', string + ' hundruð þúsund og'))
    cardinal_million_tuples.append((npa.hndrds_ptrn + number + "00" + npa.thsnds_and_ptrn_ordinal, '.*', 'hundred thousands', string + ' hundruð þúsund og'))
    cardinal_million_tuples.append((npa.hndrds_ptrn + number + npa.hndrd_and_thsnd + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', string + ' hundruð og'))
    cardinal_million_tuples.append((npa.hndrds_ptrn + number + npa.hndrd_thsnd_after + npa.dec_ptrn_ordinal, '.*', 'hundred thousands', string + ' hundruð'))